import os, logging

logger = logging.getLogger(__name__)

class Gpio:
    '''
    Module to handle with GPIOs. It allows read and write operations.
    This module works on top of GPIOs sysfs from the Linux kernel
    (https://www.kernel.org/doc/Documentation/gpio/sysfs.txt).
    '''
    GPIO_ROOT = '/sys/class/gpio'
    GPIO_EXPORT = os.path.join(GPIO_ROOT, 'export')
    gpiopath = lambda self, pin: os.path.join(self.GPIO_ROOT, 'gpio{0}'.format(pin))

    IN, OUT = 'in', 'out'

    def __init__(self, pin: str, mode: str, initial:bool=False, active_low:bool=True):
        '''
        Instantiate a GPIO object.

        :param str pin: The GPIO indentifier. Normally, a string in the form of 'gpioN', e.g., gpio19.
        :param str mode: 'in' to configure the GPIO as an input. 'out' to configure the GPIO as an output.
        :param bool initial: Initial value for the GPIO if configured as an output.
        :param bool active_low: True, to invert the value attribute for both reading and writing operations.

        Raises:
            TypeError: if 'pin' is not a string
            ValueError: if 'mode' value is not supported
        '''
        if not isinstance(pin, str):
            raise TypeError(pin)
        self._pin = pin

        if mode not in (self.IN, self.OUT):
            raise ValueError(mode)
        self._mode = mode

        ppath = self.gpiopath(pin)
        if not os.path.exists(ppath):
            logger.debug('Creating GPIO {0}'.format(pin))
            with open(self.GPIO_EXPORT, 'w') as f:
                f.write(pin)

        with open(os.path.join(ppath, 'direction'), 'w+') as f:
            f.write(mode)

        if mode == self.OUT:
            fileMode = 'w+'
        else:
            fileMode = 'r'
        try:
            self._value = open(os.path.join(ppath, 'value'), fileMode)
        except Exception as e:
            if self._value: self._value.close()
            raise e

        if mode == self.OUT:
            with open(os.path.join(ppath, 'active_low'), 'w+') as f:
                if active_low:
                    f.write('1')
                else:
                    f.write('0')

            if initial:
                self.write(1)
            else:
                self.write(0)

    @property
    def fileno(self) -> int:
        '''
        Return the 'value' file descriptor of the underlying kernel sysfs.

        :rtype: int
        :return: The file descriptor integer.
        '''
        return self._value.fileno()

    def read(self) -> int:
        '''
        Perform a read operation of the GPIO value.

        :rtype: int
        :return: The current value for the GPIO.
        '''
        self._value.seek(0)
        val = int(self._value.read().strip())
        #logger.debug('Read {0}: {1}'.format(self._pin, val))
        return val

    def write(self, value:int):
        '''
        Perform a write operation on GPIO value. Only for 'in' GPIOs.

        :param int value: Value to write in the GPIO.
        '''
        if self._mode == self.IN:
            return
        value = int(bool(value))
        logger.debug('Write {0}: {1}'.format(self._pin, value))
        self._value.write(str(value))
        self._value.flush()

